Package: munsellinterpol
Type: Package
Title: Interpolate Munsell Renotation Data from Hue Value/Chroma to
        CIE/RGB
Version: 3.4-0
Encoding: UTF-8
Date: 2026-07-03
Authors@R: c(person(given = "Jose", family = "Gama", role = c("aut","trl"), email = "rxprtgama@gmail.com"),
        person(given = "Paul", family = "Centore", role = c("aut","cph"), email = "centore@99main.com"),
        person(given = "Glenn", family = "Davis", role = c("aut","cre"), email = "gdavis@gluonics.com")  )
Description: Methods for interpolating data in the Munsell color system following the ASTM D-1535 standard. Hues and chromas with decimal values can be interpolated and converted to/from the Munsell color system and CIE xyY, CIE XYZ, CIE Lab, CIE Luv, or RGB.  Includes ISCC-NBS color block lookup.  Based on the work by Paul Centore, "The Munsell and Kubelka-Munk Toolbox".
License: GPL (>= 3)
LazyLoad: yes
LazyData: yes
Depends: R (>= 3.5.0)
Imports: rootSolve, spacesRGB, spacesXYZ (>= 1.6-0), logger
Suggests: microbenchmark, mgcv, knitr, rmarkdown, colorSpec (>= 1.8-0),
        flextable, OxSR
Author: Jose Gama [aut, trl],
  Paul Centore [aut, cph],
  Glenn Davis [aut, cre]
Maintainer: Glenn Davis <gdavis@gluonics.com>
Repository: CRAN
NeedsCompilation: no
VignetteBuilder: knitr
BuildVignettes: yes
Packaged: 2026-07-03 03:25:11 UTC; Glenn
Date/Publication: 2026-07-03 14:30:02 UTC
