#  exportPattern("^[[:alpha:]]+")


# functions written by Jose Gama and Glenn Davis
export( LabToMunsell,   LabtoMunsell,
        LuvToMunsell,   LuvtoMunsell,
        MunsellHVC,
        MunsellToLab,
        MunsellToLuv,
        MunsellTosRGB,
        MunsellToxyY,
        MunsellToXYZ,
        sRGBtoMunsell,
        xyYtoMunsell,
        XYZtoMunsell,
        IsWithinMacAdamLimits,
        MunsellSpecToColorlabFormat,
        ColorlabFormatToMunsellSpec
        )

# functions by Jose Gama, and used in package 'colorscience'
#export( # srgb2xyz,
#        xyz2srgb )

# functions added by Glenn Davis
export( HVCfromMunsellName,
        HueStringFromNumber,
        MunsellNameFromHVC,
        HueNumberFromString,
        VfromY,
        YfromV,
        plotLociHC,
        plotPatchesH,
        ColorBlockFromMunsell,
#       sectionOptimals,
        RGBtoMunsell,
        MunsellToRGB,
        NickersonColorDifference,
        roundHVC
        )

#   export for testing only
# export( process_white )

import( logger )

importFrom(rootSolve,multiroot)
#   importFrom(arrangements,combinations)
#   importFrom(geometry, delaunayn, tsearchn)
importFrom(stats, approx, spline, splinefun, predict, residuals, formula, lm, polym, coef )
importFrom(graphics, abline, grid, lines, polygon, par, plot,
                plot.new, plot.window, points, rect, segments, text, title)
importFrom(utils, data, read.table, str, flush.console, object.size, combn )
