The Munsell xy data was copied from:

http://www.rit-mcsl.org/MunsellRenotation/all.dat
http://www.rit-mcsl.org/MunsellRenotation/real.dat

Also see:
http://www.rit.edu/science/pocs/renotation


The xy data for Value >= 1 is taken from:

    Newhall, Sidney M., Dorothy Nickerson, Deane B. Judd.
    Final Report of the O.S.A. Subcommitte on the Spacing of the Munsell Colors.
    Journal of the Optical Society of America.
    Vol. 33.  No. 7.  pp. 385-418.
    July 1943.

The xy data for Value < 1 is taken from:

    Judd, Deane B. and Gunter Wyszecki.
    Extension of the Munsell Renotation System to Very Dark Colors.
    Journal of the Optical Society of America.
    Vol. 46.  No. 4.  pp. 281-284.
    April 1956.
    
In addition, the file all.dat has extrapolated xy data
that goes to higher Chromas than the data in these 2 references.
Unfortunately the method of extrapolation is not documented.
