

% right arrow, in either format
\newcommand{\rarrow}{ \if{latex}{\out{$\rightarrow$}} \if{html}{\out{&rarr;}} }

% inner-product angular brackets, in either format
\newcommand{\langle}{ \if{latex}{\out{$\langle$}} \if{html}{\out{&langle;}} }
\newcommand{\rangle}{ \if{latex}{\out{$\rangle$}} \if{html}{\out{&rangle;}} }

\newcommand{\MKMTB}{https://www.munsellcolorscienceforpainters.com/MunsellAndKubelkaMunkToolbox/MunsellAndKubelkaMunkToolbox.html}